SystemOrganization addCategory: #'Sight-Coverage'!
SystemOrganization addCategory: #'Sight-Coverage-Tests'!

SMethodExtension subclass: #SCoverageDataForMethods
	instanceVariableNames: 'tests'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Coverage'!

!SCoverageDataForMethods methodsFor: 'callbacks' stamp: 'mt 1/31/2012 11:56:22.139'!
beforeMethodWithReceiver: anObject 

	self
		calledByTest: SCurrentTest value
		inClass: anObject class.! !

!SCoverageDataForMethods methodsFor: 'callbacks' stamp: 'mt 2/27/2012 17:02:32.139'!
calledByTest: aTestReference inClass: aClass
	"The class indicates the concrete receiver class."
	
	"No support for concurrent method calls yet."
	aTestReference ifNil: [^ self].
	
	self tests add: aTestReference.
	
	aTestReference compiledMethod methodExtension "SCoverageDataForTests"
		coveredMethod: self clientMethod
		inClass: aClass
		withTest: aTestReference.! !

!SCoverageDataForMethods methodsFor: 'accessing' stamp: 'mt 1/31/2012 11:05:07.4'!
tests
	"Covering tests of this method."
	
	^ tests ifNil: [tests := KeyedIdentitySet keyBlock:
		[:testReference | testReference scaledIdentityHash]]! !

!SCoverageDataForMethods methodsFor: 'accessing' stamp: 'mt 1/31/2012 11:02:40.312'!
tests: anObject
	tests := anObject! !

SMethodExtension subclass: #SCoverageDataForTests
	instanceVariableNames: 'coverage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Coverage'!

!SCoverageDataForTests methodsFor: 'accessing' stamp: 'mt 2/9/2012 22:13:27.341'!
coverage
	"Coverage data for each possible test reference."

	^ coverage ifNil: [coverage := IdentityDictionary new]! !

!SCoverageDataForTests methodsFor: 'accessing' stamp: 'mt 1/31/2012 11:13:39.516'!
coverage: anObject
	coverage := anObject! !

!SCoverageDataForTests methodsFor: 'queries - private' stamp: 'mt 1/31/2012 14:49:10.459'!
coverageForTest: aTestReference

	^ self coverage
		at: aTestReference
		ifAbsentPut: [OrderedKeysIdentityDictionary new]! !

!SCoverageDataForTests methodsFor: 'callbacks' stamp: 'mt 1/31/2012 11:30:08.302'!
coveredMethod: aMethod inClass: aClass withTest: aTestReference

	| statistic |
	statistic := (self coverageForTest: aTestReference)
		at: aMethod
		ifAbsentPut: [KeyedIdentitySet keyBlock: [:cls | cls scaledIdentityHash]].
	statistic add: aClass.
		! !

!SCoverageDataForTests methodsFor: 'queries' stamp: 'mt 1/31/2012 11:32:53.511'!
receiverClassesForTest: aTestReference inMethod: aMethod

	^ (self coverageForTest: aTestReference)
		at: aMethod! !

Object subclass: #SCoverageCollector
	instanceVariableNames: 'result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Coverage'!

!SCoverageCollector class methodsFor: 'as yet unclassified' stamp: 'mt 1/31/2012 13:56:47.394'!
updateCoverageForPackage: aPackageName

	self
		updateCoverageForPackage: aPackageName
		testPackage: aPackageName.! !

!SCoverageCollector class methodsFor: 'as yet unclassified' stamp: 'mt 1/31/2012 13:57:48.218'!
updateCoverageForPackage: aPackageName testPackage: aTestPackageName

	| collector instrumentor package testPackage |
	package := PackageInfo named: aPackageName.
	testPackage := PackageInfo named: aTestPackageName.
	
	collector := SCoverageCollector new.
	instrumentor := SWrappingInstrumentor new keepExtensions: true.
	instrumentor createExtensionBlock: collector createExtensionBlock.
	
	instrumentor instrumentPackage: package.
	instrumentor instrumentPackage: testPackage.

	collector collectCoverage: (testPackage classes
		select: [:c | c inheritsFrom: TestCase]).

	instrumentor uninstrumentPackage: testPackage.
	instrumentor uninstrumentPackage: package.! !

!SCoverageCollector methodsFor: 'running - private' stamp: 'mt 1/31/2012 11:54:04.004'!
collectCoverage: someTestCaseClasses

	| aSuite |
	someTestCaseClasses ifEmpty: [^ self].
	
	aSuite := TestSuite new.
	someTestCaseClasses
		reject: [:tc | tc isAbstract]
		thenDo: [:tc | tc addToSuiteFromSelectors: aSuite].

	self runSuite: aSuite.! !

!SCoverageCollector methodsFor: 'running' stamp: 'mt 1/31/2012 14:26:16.123'!
collectCoverageForPackage: aPackageName

	self
		collectCoverageForPackage: aPackageName
		testPackage: aPackageName.! !

!SCoverageCollector methodsFor: 'running' stamp: 'mt 1/31/2012 14:44:08.718'!
collectCoverageForPackage: aPackageName testPackage: aTestPackageName

	| instrumentor package testPackage |
	package := PackageInfo named: aPackageName.
	testPackage := PackageInfo named: aTestPackageName.
	
	instrumentor := SWrappingInstrumentor new keepExtensions: true.
	instrumentor createExtensionBlock: self createExtensionBlock.
	
	instrumentor instrumentPackage: package.
	package ~= testPackage ifTrue: [
		instrumentor instrumentPackage: testPackage].

	self collectCoverage: (testPackage classes
		select: [:c | c inheritsFrom: TestCase]).

	package ~= testPackage ifTrue: [
		instrumentor uninstrumentPackage: testPackage].

	instrumentor uninstrumentPackage: package.! !

!SCoverageCollector methodsFor: 'as yet unclassified' stamp: 'mt 1/31/2012 13:46:10.134'!
createExtensionBlock

	^ [:aMethod | aMethod isTestMethod
		ifTrue: [SCoverageDataForTests new]
		ifFalse: [SCoverageDataForMethods new]]! !

!SCoverageCollector methodsFor: 'accessing' stamp: 'mt 1/27/2012 14:39:48.629'!
result

	^ result! !

!SCoverageCollector methodsFor: 'accessing' stamp: 'mt 1/27/2012 14:40:05.283'!
result: aTestResult

	result := aTestResult.! !

!SCoverageCollector methodsFor: 'running - private' stamp: 'mt 1/31/2012 11:50:22.375'!
runSuite: aTestSuite

	self
		result: TestResult new;
		setUpSuite: aTestSuite.

	[aTestSuite tests do: [:each | self runTest: each]]
		ensure: [self tearDownSuite: aTestSuite].! !

!SCoverageCollector methodsFor: 'running - private' stamp: 'mt 1/31/2012 11:54:42.61'!
runTest: aTestCase
	"Execute the test in the given test case instance."
	
	SCurrentTest
		value: aTestCase reference
		during: [aTestCase run: self result].
! !

!SCoverageCollector methodsFor: 'running - private' stamp: 'mt 1/27/2012 12:16:21.126'!
setUpSuite: aTestSuite

	aTestSuite resources do: [ :each |
		each isAvailable ifFalse: [ each signalInitializationError ] ].! !

!SCoverageCollector methodsFor: 'running - private' stamp: 'mt 1/27/2012 12:16:05.455'!
tearDownSuite: aTestSuite

	aTestSuite resources do: [ :each | each reset ].! !

Object subclass: #STestOrganizer
	instanceVariableNames: 'tests'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Coverage'!

STestOrganizer class
	instanceVariableNames: 'default'!

!STestOrganizer commentStamp: 'mt 2/9/2012 21:52:36.653' prior: 0!
Like the PackageOrganizer, this organizes instances of STestReference.!

STestOrganizer class
	instanceVariableNames: 'default'!

!STestOrganizer class methodsFor: 'as yet unclassified' stamp: 'mt 2/9/2012 21:53:00.357'!
default

	^ default ifNil: [default := self new]! !

!STestOrganizer methodsFor: 'queries' stamp: 'mt 2/9/2012 22:29:02.672'!
class: aTestCaseClass selector: aTestSelector ifAbsent: exceptionBlock

	^ (tests at: aTestCaseClass ifAbsent: [^ exceptionBlock value])
		at: aTestSelector ifAbsent: exceptionBlock! !

!STestOrganizer methodsFor: 'initialization' stamp: 'mt 2/9/2012 22:16:12.542'!
initialize

	super initialize.
	tests := IdentityDictionary new.! !

!STestOrganizer methodsFor: 'registering' stamp: 'mt 2/9/2012 22:19:43.913'!
registerTest: aTestReference

	(tests
		at: aTestReference testClass
		ifAbsentPut: [IdentityDictionary new])
		at: aTestReference testSelector
		put: aTestReference.! !

!STestOrganizer methodsFor: 'registering' stamp: 'mt 2/9/2012 22:21:43.232'!
unregisterTest: aTestReference

	(tests
		at: aTestReference testClass
		ifAbsent: [^ self])
		removeKey: aTestReference testSelector
		ifAbsent: [].
	
	"Clean-up."
	(tests at: aTestReference testClass)
		ifEmpty: [tests removeKey: aTestReference testClass].! !

Object subclass: #STestReference
	instanceVariableNames: 'testClass testSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Coverage'!

!STestReference commentStamp: 'mt 1/30/2012 15:07:01.559' prior: 0!
This is like a method reference but it is still valid if the #testSelector is implemented in a super class of #testClass.!

!STestReference class methodsFor: 'instance creation' stamp: 'mt 2/9/2012 22:26:39.64'!
class: aTestCaseClass selector: aTestSelector

	| ref |
	ref :=	STestOrganizer default
		class: aTestCaseClass 
		selector: aTestSelector
		ifAbsent: [].
		
	ref ifNil: [
		ref := self new
			testClass: aTestCaseClass;
			testSelector: aTestSelector.
		ref isValid
			ifTrue: [STestOrganizer default registerTest: ref]
			ifFalse: [Error signal: 'Test reference is not valid!!']].
		
	^ ref! !

!STestReference methodsFor: 'comparing' stamp: 'mt 1/30/2012 15:40:39.502'!
= anotherTestReference

	^ self species == anotherTestReference species
		and: [self testClass == anotherTestReference testClass]
		and: [self testSelector == anotherTestReference testSelector]! !

!STestReference methodsFor: 'converting' stamp: 'mt 1/31/2012 09:28:57.775'!
compiledMethod

	(self testClass includesSelector: self testSelector)
		ifTrue: [^ self testClass >> self testSelector].
		
	self testClass shouldInheritSelectors
		ifTrue: [self testClass allSuperclassesDo: [:superclass |
			(superclass includesSelector: self testSelector)
				ifTrue: [^ superclass >> self testSelector]]].
			
	Error signal: 'Test method could not be found!!'.
! !

!STestReference methodsFor: 'coverage' stamp: 'mt 2/1/2012 10:41:46.689'!
coveredMethods

	self isValid ifFalse: [^ #()].
	self compiledMethod hasMethodExtension ifFalse: [^ #()].
	
	^ (self compiledMethod methodExtension coverageForTest: self) keys! !

!STestReference methodsFor: 'comparing' stamp: 'mt 1/30/2012 15:41:14.898'!
hash

	^ (self species hash bitXor: self testClass hash) bitXor: self testSelector hash! !

!STestReference methodsFor: 'testing' stamp: 'mt 1/30/2012 15:20:02.462'!
isValid
	"A test reference is valid if the class implements the selector or if a test runner would consider subclasses that reuse that selector."

	^ (self testClass includesSelector: self testSelector)
		or: (self testClass shouldInheritSelectors
			and: [self testClass canUnderstand: self testSelector])! !

!STestReference methodsFor: 'converting' stamp: 'mt 2/16/2012 11:15:20.208'!
methodReference

	^ self compiledMethod methodReference! !

!STestReference methodsFor: 'printing' stamp: 'mt 4/7/2012 17:27:09.436'!
printString

	^ super printString, ' (', self testClass name, '>>', self testSelector, ')'! !

!STestReference methodsFor: 'running' stamp: 'mt 2/16/2012 11:12:08.47'!
run

	^ self testClass run: self testSelector! !

!STestReference methodsFor: 'accessing' stamp: 'mt 1/27/2012 11:53:22.274'!
testClass
	^ testClass! !

!STestReference methodsFor: 'accessing' stamp: 'mt 1/27/2012 11:53:22.303'!
testClass: anObject
	testClass := anObject! !

!STestReference methodsFor: 'accessing' stamp: 'mt 1/27/2012 11:53:22.32'!
testSelector
	^ testSelector! !

!STestReference methodsFor: 'accessing' stamp: 'mt 1/27/2012 11:53:22.332'!
testSelector: anObject
	testSelector := anObject! !

STestReference subclass: #SUncachedTestReference
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Coverage-Tests'!

!SUncachedTestReference class methodsFor: 'instance creation' stamp: 'mt 2/9/2012 22:30:29.661'!
class: aTestCaseClass selector: aTestSelector

	^ self new
		testClass: aTestCaseClass;
		testSelector: aTestSelector! !

!SUncachedTestReference methodsFor: 'private' stamp: 'mt 2/9/2012 22:33:23.965'!
species

	^ STestReference! !

TestCase subclass: #SFakeTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Coverage-Tests'!

!SFakeTests commentStamp: 'mt 1/31/2012 09:26:23.669' prior: 0!
Used for STestReferenceTests.!

SFakeTests subclass: #SFakeOwnSubclassedTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Coverage-Tests'!

!SFakeOwnSubclassedTests commentStamp: 'mt 1/31/2012 09:43:08.101' prior: 0!
Does not inherit tests from superclass.!

!SFakeOwnSubclassedTests methodsFor: 'fake tests' stamp: 'mt 1/31/2012 09:43:59.774'!
test3! !

SFakeTests subclass: #SFakeSubclassedTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Coverage-Tests'!

!SFakeSubclassedTests class methodsFor: 'testing' stamp: 'mt 1/31/2012 09:26:56.592'!
shouldInheritSelectors

	^ true! !

!SFakeSubclassedTests methodsFor: 'fake tests' stamp: 'mt 1/31/2012 09:27:26.792'!
test2

! !

!SFakeTests methodsFor: 'fake tests' stamp: 'mt 1/31/2012 09:27:44.686'!
test1! !

TestCase subclass: #STestOrganizerTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Coverage-Tests'!

!STestOrganizerTests methodsFor: 'tests' stamp: 'mt 2/9/2012 22:39:44.895'!
test01Register

	| sut |
	sut := STestOrganizer new.
	
	self assert: (sut class: SFakeTests selector: #test1 ifAbsent: []) isNil.
	
	sut registerTest: (SUncachedTestReference class: SFakeTests selector: #test1).
	
	self assert: (sut class: SFakeTests selector: #test1 ifAbsent: []) isValid.! !

!STestOrganizerTests methodsFor: 'tests' stamp: 'mt 2/9/2012 22:40:04.01'!
test02Unregister

	| sut |
	sut := STestOrganizer new.
	sut registerTest: (SUncachedTestReference class: SFakeTests selector: #test1).
	
	self assert: (sut class: SFakeTests selector: #test1 ifAbsent: []) isValid.
	
	sut unregisterTest: (sut class: SFakeTests selector: #test1 ifAbsent: []).
	
	self assert: (sut class: SFakeTests selector: #test1 ifAbsent: []) isNil.
	
	
	
	! !

!STestOrganizerTests methodsFor: 'tests' stamp: 'mt 2/9/2012 22:41:19.492'!
test03Identity

	| ref1 ref2 |
	ref1 := STestReference class: SFakeTests selector: #test1.
	ref2 := STestReference class: SFakeTests selector: #test1.
	self assert: ref1 == ref2.! !

TestCase subclass: #STestReferenceTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Coverage-Tests'!

!STestReferenceTests methodsFor: 'tests' stamp: 'mt 2/9/2012 22:30:52.265'!
test01Creation

	| ref |
	ref := SUncachedTestReference class: SFakeTests selector: #test1.
	
	self
		assert: SFakeTests equals: ref testClass;
		assert: #test1 equals: ref testSelector.! !

!STestReferenceTests methodsFor: 'tests' stamp: 'mt 2/9/2012 22:31:05.736'!
test02Validation

	| validRef invalidRef |
	validRef := SUncachedTestReference class: SFakeTests selector: #test1.
	invalidRef := SUncachedTestReference class: SFakeTests selector: #testX.
	
	self
		assert: validRef isValid;
		assert: invalidRef isValid not.! !

!STestReferenceTests methodsFor: 'tests' stamp: 'mt 2/9/2012 22:31:14.831'!
test03Comparison
	"Create two references to the same test and check if they are equal."

	| ref1 ref2 |
	ref1 := SUncachedTestReference class: SFakeTests selector: #test1.
	ref2 := SUncachedTestReference class: SFakeTests selector: #test1.
	
	self
		assert: ref1 ~~ ref2;
		assert: ref1 = ref2.! !

!STestReferenceTests methodsFor: 'tests' stamp: 'mt 2/9/2012 22:31:20.772'!
test04Method
	"Check if the reference points to the correct compiled method."
	
	| ref |
	ref := SUncachedTestReference class: SFakeTests selector: #test1.
	
	self assert: ref compiledMethod == (SFakeTests >> #test1).! !

!STestReferenceTests methodsFor: 'tests' stamp: 'mt 2/9/2012 22:31:27.374'!
test05MethodError
	"Invalid references should raise an error when trying to access a compiled method."

	| ref |
	ref := SUncachedTestReference class: SFakeTests selector: #testX.
	
	self should: [ref compiledMethod] raise: Error.! !

!STestReferenceTests methodsFor: 'tests' stamp: 'mt 2/9/2012 22:31:34.397'!
test06CreationFromTestCase

	| testCase ref |
	testCase := SFakeTests selector: #test1.
	ref := SUncachedTestReference class: SFakeTests selector: #test1.
	
	self assert: ref = testCase reference.! !

!STestReferenceTests methodsFor: 'tests' stamp: 'mt 2/9/2012 22:31:43.389'!
test07InheritedTest

	| ref |
	ref := SUncachedTestReference class: SFakeSubclassedTests selector: #test1.
	
	self
		assert: ref isValid;
		assert: ref compiledMethod == (SFakeTests >> #test1).! !

!STestReferenceTests methodsFor: 'tests' stamp: 'mt 2/9/2012 22:31:48.329'!
test08InheritedTestInvalid
	"What happens if #shouldInheritSelectors is false? The reference to a base class test becomes invalid."

	| ref |
	ref := SUncachedTestReference class: SFakeOwnSubclassedTests selector: #test1.	
	self assert: ref isValid not.! !

!TestCase methodsFor: '*Sight-Coverage' stamp: 'mt 1/27/2012 15:10:49.817'!
reference

	^ STestReference
		class: self class
		selector: self selector! !

DynamicVariable subclass: #SCurrentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Coverage'!
